/* chkmail.c -- Asus M6N Email LED control program
   Copyright (C) 2004 Howard Chu - hyc <at> highlandsun.com

This file is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version
2 of the License, or (at your option) any later version.

This file is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

A copy of the license is available on the web at
http://www.fsf.org/licenses/gpl.html

*/

/* This program only works on Windows XP. It talks to the
ASUS ATKACPI driver, so you must have that already installed.
The program monitors the registry keys that are used by the
SHGetUnreadMailCount function for any changes and updates
the Email LED accordingly. */

#include <windows.h>
#include <shlobj.h>

static const char info[] =
	"Asus M6N ChkMail by Howard Chu hyc <at> highlandsun.com";

HRESULT WINAPI SHGetUnreadMailCountW(
	void *hKeyUser,
	void *MailAddress,
	void *count,
	FILETIME *stamp,
	void *Shellcommand,
	int doShell);

HANDLE ATKACPIhandle;

/* There are several magic values that the Asus driver recognizes.
 MLED is the only one we care about here. */

#define MLED	0x44454c4d

int CtrlACPI(int code, int hasArg, int arg)
{
	long bytes = 0;
	long inbuf[5];
	struct cmbuf {
		short cmds[2];
		long cm2;
	} cbuf;
	long outbuf[192];
	int ret;
	
	cbuf.cmds[0] = 0;
	cbuf.cmds[1] = 4;
	cbuf.cm2 = arg;
	inbuf[0] = 2;
	inbuf[1] = code;
	inbuf[2] = hasArg;
	inbuf[3] = 8 * hasArg;
	inbuf[4] = (long)&cbuf;

	ATKACPIhandle = CreateFile("\\\\.\\ATKACPI",
			GENERIC_READ|GENERIC_WRITE,
			FILE_SHARE_READ|FILE_SHARE_WRITE, NULL, OPEN_EXISTING,
			0, NULL);

	if (!ATKACPIhandle) return -1;

	ret = DeviceIoControl(ATKACPIhandle, 0x222404, inbuf, sizeof(inbuf),
		outbuf, sizeof(outbuf), &bytes, NULL);

	CloseHandle(ATKACPIhandle);

	return ret;
}

/* Bare minimum Window processing */

WNDCLASS wndclass;

LRESULT CALLBACK wndproc(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	int ret;
	switch(uMsg) {
	case WM_DESTROY:
		PostQuitMessage(0);
		ret = 0;
		break;
	default:
		ret = DefWindowProc(hwnd,uMsg,wParam,lParam);
		break;
	}
	return ret;
}

int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInst, PSTR cmdLine,
	int iCmdShow)
{
	FILETIME stamp = {0};
	long old = -1, count;
	HWND hwnd;
	HKEY UnreadMailKey;
	HANDLE KeyEvent;

	/* We create a window just so that we can be closed gracefully.
	   We never do any drawing or anything else... */

	wndclass.hInstance = hInstance;
	wndclass.lpszClassName = "hyc";
	wndclass.lpfnWndProc = wndproc;

	/* A do-once "loop." It's just a convenience that lets us use
	   "break" to bail out on errors. */
	do {

		/* Set up our main window */
		if (RegisterClass(&wndclass) == 0 ) break;

		hwnd = CreateWindow("hyc", "hyc", WS_OVERLAPPEDWINDOW,
			CW_USEDEFAULT, CW_USEDEFAULT,
			CW_USEDEFAULT, CW_USEDEFAULT,
			NULL, NULL, wndclass.hInstance, NULL);

		if (!hwnd) break;

		/* Open the Asus device driver. If it's not there,
		   there's nothing we can do. */
		ATKACPIhandle = CreateFile("\\\\.\\ATKACPI",
			GENERIC_READ|GENERIC_WRITE,
			FILE_SHARE_READ|FILE_SHARE_WRITE, NULL, OPEN_EXISTING,
			0, NULL);

		if (!ATKACPIhandle) break;

		CloseHandle(ATKACPIhandle);

		/* Get the registry key for the current user's Unread Mail
		   counter. It only exists on Windows XP. */
		if (RegOpenKeyEx(HKEY_CURRENT_USER,"Software\\Microsoft\\Windows\\CurrentVersion\\UnreadMail", 0, KEY_READ, &UnreadMailKey)) break;

		/* Create an event that will be used to notify us whenever
		   the counter is changed. */
		KeyEvent = CreateEvent(NULL, FALSE, FALSE, NULL);
		if (!KeyEvent) break;

		for (;;) {
			int ret;

			/* Get the unread count. If it's not the same as the
			   last time we came through here, update the LED. */
			SHGetUnreadMailCountW(NULL, NULL, &count, &stamp, NULL, 0);
			if ( count != old ) {
				old = count;

				/* arg: 0 = on, 1 = off */
				CtrlACPI(MLED, 1, count == 0);
			}

			/* Set up event notification on changes to our key */
			if (RegNotifyChangeKeyValue(UnreadMailKey, TRUE,
				REG_NOTIFY_CHANGE_NAME|REG_NOTIFY_CHANGE_LAST_SET,
				KeyEvent, TRUE)) break;

			/* Wait for the key to change or any Windows messages
			   to arrive */
			if ((ret=MsgWaitForMultipleObjects(1, &KeyEvent, FALSE,
				INFINITE, QS_ALLINPUT)) > WAIT_OBJECT_0) {
				MSG msg;

				/* See if we're quitting, otherwise just
				   process a message */
				if ((ret=GetMessage(&msg, hwnd, 0, 0))==0 ||
					ret == -1) break;
				TranslateMessage(&msg);
				DispatchMessage(&msg);
			}
		}
	} while(0);
}
